package edu.uoregon.tau.paraprof.treetable;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.ListModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.uoregon.tau.common.ImageExport;
import edu.uoregon.tau.common.treetable.AbstractTreeTableModel;
import edu.uoregon.tau.common.treetable.JTreeTable;
import edu.uoregon.tau.paraprof.ColorBar;
import edu.uoregon.tau.paraprof.ParaProf;
import edu.uoregon.tau.paraprof.ParaProfException;
import edu.uoregon.tau.paraprof.ParaProfTrial;
import edu.uoregon.tau.paraprof.ParaProfUtils;
import edu.uoregon.tau.paraprof.SearchPanel;
import edu.uoregon.tau.paraprof.WindowPlacer;
import edu.uoregon.tau.paraprof.interfaces.ParaProfWindow;
import edu.uoregon.tau.paraprof.interfaces.Searchable;
import edu.uoregon.tau.paraprof.interfaces.SearchableOwner;
import edu.uoregon.tau.paraprof.interfaces.UnitListener;
import edu.uoregon.tau.paraprof.treetable.ColumnChooser.CheckBoxListItem;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.ExclusiveColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.ExclusivePerCallColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.ExclusivePercentColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.InclusiveColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.InclusivePerCallColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.InclusivePercentColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.NumCallsColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.NumSubrColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.RegularMetricColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.RegularPerCallMetricColumn;
import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.RegularPercentMetricColumn;
import edu.uoregon.tau.perfdmf.Function;
import edu.uoregon.tau.perfdmf.Thread;

/**
 * Displays callpath data in a Cube style tree/table.  Non-callpath data is also displayed as a flat tree.
 * This is a replacement for the statistics text window.  It shows all that data and more, plus allows easy
 * sorting by column, reordering of columns, and choosing of columns.
 *    
 * TODO : ...
 *
 * <P>CVS $Id: TreeTableWindow.java,v 1.18 2010/03/16 02:16:28 amorris Exp $</P>
 * @author  Alan Morris
 * @version $Revision: 1.18 $
 */
public class TreeTableWindow extends JFrame implements TreeExpansionListener, Observer, ParaProfWindow, Printable, UnitListener,
        ImageExport, SearchableOwner, ActionListener, KeyListener, Searchable {

    /**
	 * 
	 */
	private static final long serialVersionUID = 4726597182083786681L;
	private ParaProfTrial ppTrial;
    private Thread thread;

    private CallPathModel model;
    private JTreeTable treeTable;
    private int colorMetricID;
    private int units = ParaProf.preferences.getUnits();
    private int decimals = -1; // -1 = auto, -2 = full


    private List<TreeTableColumn> columns;
    private ColumnChooser columnChooser;
    
    private JCheckBoxMenuItem showFindPanelBox= new JCheckBoxMenuItem("Show Find Panel", false);
    private SearchPanel searchPanel;

    private final JMenuItem showAsTreeMenuItem = new JCheckBoxMenuItem("Show as Call Tree");
    private final JMenuItem reverseTreeMenuItem = new JCheckBoxMenuItem("Reverse Call Tree", false);
    private final JMenuItem showInclExclMenuItem = new JCheckBoxMenuItem("Show Inclusive/Exclusive", true);
    private JScrollPane scrollPane;

    public TreeTableWindow(ParaProfTrial ppTrial, Thread thread) {
        this(ppTrial, thread, null);
    }

    public TreeTableWindow(ParaProfTrial ppTrial, Thread thread, Component invoker) {

        this.ppTrial = ppTrial;
        this.thread = thread;
        ppTrial.addObserver(this);

        if (!ppTrial.getDataSource().getReverseDataAvailable()) {
            reverseTreeMenuItem.setEnabled(false);
        }

        // create the column chooser.  Note: the column chooser holds the data on which columns are shown
        columnChooser = new ColumnChooser(this, ppTrial);

        setSize(ParaProfUtils.checkSize(new Dimension(1000, 600)));

        setLocation(WindowPlacer.getNewLocation(this, invoker));

        if (thread.getNodeID() == -1 || thread.getNodeID() == -6) {
            this.setTitle("TAU: ParaProf: Mean Statistics - "
                    + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
        } else if (thread.getNodeID() == -2) {
            this.setTitle("TAU: ParaProf: Total Statistics - "
                    + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
        } else if (thread.getNodeID() == -3 || thread.getNodeID() == -7) {
            this.setTitle("TAU: ParaProf: Std. Dev. Statistics - "
                    + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
        } else {
            this.setTitle("TAU: ParaProf: Statistics for: " + ParaProfUtils.getThreadLabel(thread) //"n,c,t, " + thread.getNodeID() + "," + thread.getContextID()  + "," + thread.getThreadID() 
            		+ " - "+ ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
        }
        ParaProfUtils.setFrameIcon(this);

        if (ParaProf.getHelpWindow().isVisible()) {
            this.help(false);
        }

        setupMenus();
        setupData();

        ParaProf.incrementNumWindows();
    }

    public void updateColumns() {

        setColumns();

        AbstractTableModel atm = (AbstractTableModel) treeTable.getModel();
        atm.fireTableChanged(new TableModelEvent(atm, TableModelEvent.HEADER_ROW));

        // reset the cell rendereres
        for (int i = 0; i < columns.size(); i++) {
            treeTable.setDefaultRenderer(columns.get(i).getClass(), columns.get(i).getCellRenderer());
        }
    }

    // get the chosen columns from the columnChooser and setup the 'columns' List
    private void setColumns() {
        columns = new ArrayList<TreeTableColumn>();

        ListModel metricModel = columnChooser.getMetricModel();
        ListModel valueModel = columnChooser.getValueModel();

        for (int i = 0; i < metricModel.getSize() - 2; i++) { // -2 because the last two are calls and subr
            CheckBoxListItem item = (CheckBoxListItem) metricModel.getElementAt(i);
            if (item.getSelected()) {

                boolean absSet = false;
                boolean percentSet = false;
                boolean percallSet = false;
                for (int j = 0; j < valueModel.getSize(); j++) {
                    CheckBoxListItem valueItem = (CheckBoxListItem) valueModel.getElementAt(j);
                    if (valueItem.getSelected()) {
                        String str = (String) valueItem.getUserObject();
                        if (str.equals("Exclusive Value") || str.equals("Inclusive Value")) {
                            if (showInclExclMenuItem.isSelected()) {
                                if (str.equals("Inclusive Value")) {
                                    columns.add(new InclusiveColumn(this, i));
                                }
                                if (str.equals("Exclusive Value")) {
                                    columns.add(new ExclusiveColumn(this, i));
                                }
                            } else {
                                if (absSet == false) {
                                    columns.add(new RegularMetricColumn(this, i));
                                    absSet = true;
                                }
                            }
                        } else if (str.equals("Inclusive Percent Value") || str.equals("Exclusive Percent Value")) {
                            if (showInclExclMenuItem.isSelected()) {
                                if (str.equals("Inclusive Percent Value")) {
                                    columns.add(new InclusivePercentColumn(this, i));
                                }
                                if (str.equals("Exclusive Percent Value")) {
                                    columns.add(new ExclusivePercentColumn(this, i));
                                }
                            } else {
                                if (percentSet == false) {
                                    columns.add(new RegularPercentMetricColumn(this, i));
                                    percentSet = true;
                                }
                            }
                        } else if (str.equals("Inclusive Value Per Call") || str.equals("Exclusive Value Per Call")) {
                            if (showInclExclMenuItem.isSelected()) {
                                if (str.equals("Inclusive Value Per Call")) {
                                    columns.add(new InclusivePerCallColumn(this, i));
                                }
                                if (str.equals("Exclusive Value Per Call")) {
                                    columns.add(new ExclusivePerCallColumn(this, i));
                                }
                            } else {
                                if (percallSet == false) {
                                    columns.add(new RegularPerCallMetricColumn(this, i));
                                    percallSet = true;
                                }
                            }
                        }
                    }
                }
            }
        }

        CheckBoxListItem callitem = (CheckBoxListItem) metricModel.getElementAt(metricModel.getSize() - 2);
        CheckBoxListItem subritem = (CheckBoxListItem) metricModel.getElementAt(metricModel.getSize() - 1);

        if (callitem.getSelected()) {
            columns.add(new NumCallsColumn(this));
        }

        if (subritem.getSelected()) {
            columns.add(new NumSubrColumn(this));
        }
    }

    private void addComponents() {

        getContentPane().removeAll();
        getContentPane().setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();

        if (showAsTreeMenuItem.isSelected() == false) {
            showInclExclMenuItem.setEnabled(false);
            showInclExclMenuItem.setSelected(true);
        } else {
            showInclExclMenuItem.setEnabled(true);

            gbc.anchor = GridBagConstraints.NORTH;
            gbc.weighty = 0.0;

            if (ppTrial.getNumberOfMetrics() > 1) {
                final JComboBox metricBox = new JComboBox(ppTrial.getMetricArray());
                metricBox.setSelectedItem(ppTrial.getDefaultMetric());

                metricBox.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        model.setColorMetric(metricBox.getSelectedIndex());
                        repaint();
                    }
                });
                gbc.weightx = 0.0;
                gbc.fill = GridBagConstraints.NONE;
                addCompItem(metricBox, gbc, 0, 0, 1, 1);
            }

            gbc.weightx = 1.0;
            gbc.fill = GridBagConstraints.HORIZONTAL;
            addCompItem(new ColorBar(), gbc, 1, 0, 1, 1);
        }

        if (scrollPane != null) {
            getContentPane().remove(scrollPane);
        }

        scrollPane = new JScrollPane(treeTable);

        for (int i = 0; i < columns.size(); i++) {
            treeTable.setDefaultRenderer(columns.get(i).getClass(), columns.get(i).getCellRenderer());
        }

        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.SOUTH;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        addCompItem(scrollPane, gbc, 0, 1, GridBagConstraints.REMAINDER,1);

        validate();
    }

    private void setupData() {
        setColumns();

        //columns.add(new StdDevColumn(this, 0));
        //columns.add(new MiniHistogramColumn(this));

        model = new CallPathModel(this, ppTrial, thread, reverseTreeMenuItem.isSelected());
        createTreeTable(model);
        addComponents();
    }

    private void addCompItem(Component c, GridBagConstraints gbc, int x, int y, int w, int h) {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        getContentPane().add(c, gbc);
    }

    private void helperAddButton(JMenu menu, ActionListener actionListener, ButtonGroup group, String value, boolean selected) {
        JRadioButtonMenuItem button = new JRadioButtonMenuItem(value, selected);
        button.addActionListener(actionListener);
        group.add(button);
        menu.add(button);
    }

    private JMenu createDecimalMenu() {

        ActionListener actionListener = new ActionListener() {

            public void actionPerformed(ActionEvent evt) {
                try {

                    String arg = evt.getActionCommand();
                    if (arg.equals("auto")) {
                        decimals = -1;
                    } else if (arg.equals("display full numbers")) {
                        decimals = -2;
                    } else if (arg.equals("0 decimal places")) {
                        decimals = 0;
                    } else if (arg.equals("1 decimal places")) {
                        decimals = 1;
                    } else if (arg.equals("2 decimal places")) {
                        decimals = 2;
                    } else if (arg.equals("3 decimal places")) {
                        decimals = 3;
                    } else if (arg.equals("4 decimal places")) {
                        decimals = 4;
                    }
                    treeTable.forceRedraw();

                } catch (Exception e) {
                    ParaProfUtils.handleException(e);
                }
            }
        };

        JMenu decimalSubMenu = new JMenu("Select Precision...");
        ButtonGroup group = new ButtonGroup();

        helperAddButton(decimalSubMenu, actionListener, group, "auto", true);
        helperAddButton(decimalSubMenu, actionListener, group, "0 decimal places", false);
        helperAddButton(decimalSubMenu, actionListener, group, "1 decimal places", false);
        helperAddButton(decimalSubMenu, actionListener, group, "2 decimal places", false);
        helperAddButton(decimalSubMenu, actionListener, group, "3 decimal places", false);
        helperAddButton(decimalSubMenu, actionListener, group, "4 decimal places", false);
        helperAddButton(decimalSubMenu, actionListener, group, "display full numbers", false);

        return decimalSubMenu;
    }

    private void setupMenus() {

        JMenuBar mainMenu = new JMenuBar();
        JMenu optionsMenu = new JMenu("Options");
        
        
        showFindPanelBox.addActionListener(this);
        optionsMenu.add(showFindPanelBox);

        ActionListener actionListener = new ActionListener() {

            public void actionPerformed(ActionEvent evt) {
                try {
                    Object EventSrc = evt.getSource();
                    if (EventSrc instanceof JMenuItem) {
                        String arg = evt.getActionCommand();
                        if (arg.equals("Show as Call Tree")) {
                        	showSearchPanel(false);
                            if (showAsTreeMenuItem.isSelected()) {
                                showInclExclMenuItem.setSelected(false);
                            }
                            setupData();
                        } else if (arg.equals("Reverse Call Tree")) {
                        	showSearchPanel(false);
                            setupData();
                        } else if (arg.equals("Show Inclusive/Exclusive")) {
                        	showSearchPanel(false);
                            setupData();
                        }
                    }
                } catch (Exception e) {
                    ParaProfUtils.handleException(e);
                }
            }
        };

        showAsTreeMenuItem.addActionListener(actionListener);
        optionsMenu.add(showAsTreeMenuItem);
        showAsTreeMenuItem.setSelected(true);

        reverseTreeMenuItem.addActionListener(actionListener);
        optionsMenu.add(reverseTreeMenuItem);

        showInclExclMenuItem.addActionListener(actionListener);
        optionsMenu.add(showInclExclMenuItem);

        JMenu unitsSubMenu = ParaProfUtils.createUnitsMenu(this, units, false);
        optionsMenu.add(unitsSubMenu);

        optionsMenu.add(createDecimalMenu());

        JMenuItem menuItem = new JMenuItem("Choose Columns...");
        menuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                columnChooser.showDialog(TreeTableWindow.this, true);
            }

        });

        optionsMenu.add(menuItem);

        if (!ppTrial.callPathDataPresent()) {
            showAsTreeMenuItem.setSelected(false);
            showAsTreeMenuItem.setEnabled(false);
            showInclExclMenuItem.setSelected(true);
            showInclExclMenuItem.setEnabled(false);
        }

        mainMenu.add(ParaProfUtils.createFileMenu((ParaProfWindow) this, this, this));
        mainMenu.add(optionsMenu);
        mainMenu.add(ParaProfUtils.createWindowsMenu(ppTrial, this));
        mainMenu.add(ParaProfUtils.createHelpMenu(this, this));

        setJMenuBar(mainMenu);
    }

    private void createTreeTable(AbstractTreeTableModel model) {

        treeTable = new JTreeTable(model, true, true);

        final JTree tree = treeTable.getTree();
        FontMetrics fm = tree.getFontMetrics(ParaProf.preferences.getFont());
		tree.setRowHeight(fm.getHeight());

        // Add a mouse listener for this tree.
        MouseListener ml = new MouseAdapter() {
            public void mousePressed(MouseEvent evt) {
                try {
                    //int selRow = tree.getRowForLocation(evt.getX(), evt.getY());
                    TreePath path = tree.getPathForLocation(evt.getX(), evt.getY());
                    if (path != null) {
                        TreeTableNode node = (TreeTableNode) path.getLastPathComponent();
                        if (ParaProfUtils.rightClick(evt)) {
                            JPopupMenu popup;
                            if (node.getFunctionProfile() != null) {
                                popup = ParaProfUtils.createFunctionClickPopUp(node.getModel().getPPTrial(),
                                        node.getFunctionProfile().getFunction(), getThread(), treeTable);
                                popup.show(treeTable, evt.getX(), evt.getY());
                            } else {
                                //popup = new JPopupMenu();
                            }
                        }
                    }
                } catch (Exception e) {
                    ParaProfUtils.handleException(e);
                }
            }
        };

        treeTable.addMouseListener(ml);

        treeTable.getTree().addTreeExpansionListener(this);
        TreePortionCellRenderer renderer = new TreePortionCellRenderer();
        treeTable.getTree().setCellRenderer(renderer);

        Font font = ParaProf.preferencesWindow.getFont();
        treeTable.setFont(font);
        renderer.setFont(font);

        FontMetrics fontMetrics = getFontMetrics(font);
        treeTable.setRowHeight(fontMetrics.getMaxAscent() + fontMetrics.getMaxDescent() + 3);

        treeTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        treeTable.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);

        TableColumn col = treeTable.getColumnModel().getColumn(0);

        int nameWidth = 500;

        if (ppTrial.getNumberOfMetrics() > 1) {
            nameWidth = 350;
        }

        if (ParaProf.demoMode) {
            nameWidth = 200;
        }

        col.setPreferredWidth(nameWidth);

        int numCols = treeTable.getColumnCount();
        col = treeTable.getColumnModel().getColumn(numCols - 2);
        col.setPreferredWidth(10);
        col = treeTable.getColumnModel().getColumn(numCols - 1);
        col.setPreferredWidth(10);
    }

    public void treeCollapsed(TreeExpansionEvent event) {
        TreeTableNode node = (TreeTableNode) event.getPath().getLastPathComponent();
        node.setExpanded(false);
    }

    public void treeExpanded(TreeExpansionEvent event) {
        TreeTableNode node = (TreeTableNode) event.getPath().getLastPathComponent();
        node.setExpanded(true);
    }

    public void update(Observable o, Object arg) {
    	if(thread.getNodeID()>=0){
    		this.setTitle("TAU: ParaProf: Statistics for: " + ParaProfUtils.getThreadLabel(thread) + " - "+ ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
    	}
        setupData();
    }

    public void help(boolean display) {
        ParaProf.getHelpWindow().clearText();
        if (display) {
            ParaProf.getHelpWindow().setVisible(true);
        }
        ParaProf.getHelpWindow().writeText("This is the Statistics Table.\n");
        ParaProf.getHelpWindow().writeText("This window shows you function data across a given thread (or mean/std.dev.)\n");
        ParaProf.getHelpWindow().writeText("If callpath data is present, it will be shown as a tree on the left.");
        ParaProf.getHelpWindow().writeText(
                "In this mode, the metric values will show Inclusive when the node is collapsed, and exclusive when the node is expanded.\n");
    }

    public void closeThisWindow() {
        setVisible(false);
        ppTrial.deleteObserver(this);
        ParaProf.decrementNumWindows();
        dispose();
    }

    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
        try {
            if (pageIndex >= 1) {
                return NO_SUCH_PAGE;
            }

            ParaProfUtils.scaleForPrint(graphics, pageFormat, (int) treeTable.getSize().getWidth(),
                    (int) treeTable.getSize().getHeight());
            export((Graphics2D) graphics, false, true, false);

            return Printable.PAGE_EXISTS;
        } catch (Exception e) {
            ParaProfUtils.handleException(e);
            return NO_SUCH_PAGE;
        }
    }

    public boolean getExclusiveInclusiveMode() {
        return showInclExclMenuItem.isSelected();
    }

    public boolean getTreeMode() {
        return showAsTreeMenuItem.isSelected();
    }

    public int getColorMetricID() {
        return colorMetricID;
    }

    public void setUnits(int units) {
        this.units = units;
        treeTable.forceRedraw();
    }

    public int getUnits() {
        return units;
    }

    public List<TreeTableColumn> getColumns() {
        return columns;
    }

    public ParaProfTrial getPPTrial() {
        return ppTrial;
    }

    public void export(Graphics2D g2D, boolean toScreen, boolean fullWindow, boolean drawHeader) {
        if (fullWindow) {
            // first draw the column headers
            scrollPane.getColumnHeader().paintAll(g2D);
            // translate past the column headers
            g2D.translate(0, scrollPane.getColumnHeader().getHeight());
            // draw the entire treetable
            treeTable.paintAll(g2D);
        } else {
            scrollPane.paintAll(g2D);
        }
    }

    public Dimension getImageSize(boolean fullScreen, boolean header) {
        if (fullScreen) {
            Dimension d = treeTable.getSize();
            // we want to draw the column headers as well
            d.setSize(d.getWidth(), d.getHeight() + scrollPane.getColumnHeader().getHeight());
            return d;
        } else {
            return scrollPane.getSize();
        }
    }

    public Thread getThread() {
        return thread;
    }

    public JFrame getFrame() {
        return this;
    }
    

    public int getDecimals() {
        return decimals;
    }

    public void setDecimals(int decimals) {
        this.decimals = decimals;
    }
    
    public void select(final Function function){
    	String[] chunks = function.getName().split("=>");
    	selectStrings(chunks);
    	
    }

    public void collapseTree(){
    	JTree jTree=treeTable.getTree();
    	for (int i = 0; i < jTree.getRowCount(); i++) {
    	         jTree.collapseRow(i);
    	}
    	
    	
    }
    
    public boolean selectStrings(String[] chunks){
    	JTree tree = treeTable.getTree();
    	TreeModel treeModel=tree.getModel();
    	TreeTableNode treeRoot =(TreeTableNode) treeModel.getRoot();
    	int treeChildren = treeModel.getChildCount(treeRoot);
    	//System.out.println(treeRoot.getClass());
    	int row=0;
    	collapseTree();
    	for(int i=0;i<chunks.length;i++)
    	{
    		for(int j=0;j<treeChildren;j++){
    			TreeTableNode ttn = (TreeTableNode) treeModel.getChild(treeRoot, j);
    			//System.out.println(chunks[i]+" vs "+ttn);
    			if(chunks[i].trim().equals(ttn.toString().trim())){
    				treeRoot=ttn;
    				treeChildren=treeModel.getChildCount(treeRoot);
    				row+=j;
    				//System.out.println("expanding "+ttn.toString().trim()+" at row: "+row);
    				TreePath tp = tree.getPathForRow(row);
    				if(!tp.getLastPathComponent().toString().equals(ttn.toString())){
    					//System.out.println("Pathing error?");
    					while(!tree.getPathForRow(row).getLastPathComponent().toString().equals(ttn.toString()))
    					{
    						
    						row++;
    						if(row>=tree.getRowCount()){
    							System.out.println("Tree Search Pathing Error.");
    							return false;
    						}
    					}
    				}
    				//System.out.println(tree.getPathForRow(row)+"\n");
    				tree.expandRow(row);
    				//lastExpanded.add(row);
    				row++;
    				break;
    			}
    		}
    	}
    	if(row>0)
    	{
    		row--;
    	}
    	else{
    		return false;
    	}
    	tree.setSelectionRow(row);
    	int oneRow = treeTable.getRowHeight()+treeTable.getRowMargin();
    	treeTable.scrollRectToVisible(new Rectangle(0,oneRow*row,5,50));
    	validate();
    	return true;
    }
    
    boolean searchMatchCase=true;
    
    private class SearchNode{
    	private int searchRow=0;
    	private TreeTableNode ttn=null;
    	public SearchNode()
		{searchRow=0;
    	ttn=null;}
    	public SearchNode(int searchRow, TreeTableNode ttn) {
			super();
			this.searchRow = searchRow;
			this.ttn = ttn;
		}
		public int getSearchRow() {
			return searchRow;
		}
		public void setSearchRow(int searchRow) {
			this.searchRow = searchRow;
		}
		public TreeTableNode getTtn() {
			return ttn;
		}
		public void setTtn(TreeTableNode ttn) {
			this.ttn = ttn;
		}
		 List<Object> nodes = new ArrayList<Object>();
		 public void addNode(TreeNode node){
			 nodes.add(0,node);
		 }
//		 public TreePath getPath(){
//			 return new TreePath(nodes.toArray());
//		 }
		 public String[] getPathStrings(){
			 String[] nodeStrings = new String[nodes.size()];
			 for(int i=0;i<nodes.size();i++){
				 nodeStrings[i]=nodes.get(i).toString();
			 }
			 return nodeStrings;
		 }
    }
    
    SearchNode sn = new SearchNode();
    //Keeps track of the last node we found in case we want to skip it on a 'next' search
    TreeTableNode lastNode=null;
    //If we've hit next we don't want to clear or list of previous searches
    boolean nexting=false;
    //Keeps track of every item we've found with 'next' so far.
    List<TreeTableNode> prevNodes=new ArrayList<TreeTableNode>();
    
    public SearchNode recSearch(String find, int row, TreeTableNode root, TreeModel tm){
    	int treeChildren = tm.getChildCount(root);

    		for(int j=0;j<treeChildren;j++){
    			TreeTableNode ttn = (TreeTableNode) tm.getChild(root, j);
    			String subject=ttn.toString().trim();
    			if(!searchMatchCase){
    				subject=subject.toUpperCase();
    			}
    			if(subject.contains(find)&&!prevNodes.contains(ttn)){
    				SearchNode goodNode = new SearchNode(row+j,ttn);
    				goodNode.addNode(ttn);
    				lastNode=ttn;
    				return  goodNode;

    			}
    			SearchNode tmpNode = recSearch(find,j+row,ttn,tm);
    			if(tmpNode!=null){
    				tmpNode.addNode(ttn);
    				return tmpNode;
    			}
    			
    		}
    	return null;
    }
    
    
    public static TreePath getPath(TreeNode treeNode) {
        List<Object> nodes = new ArrayList<Object>();
        if (treeNode != null) {
          nodes.add(treeNode);
          treeNode = treeNode.getParent();
          while (treeNode != null) {
            nodes.add(0, treeNode);
            treeNode = treeNode.getParent();
          }
        }

        return nodes.isEmpty() ? null : new TreePath(nodes.toArray());
      }
    
    public boolean search(String find){
    	boolean found=false;
    	JTree tree = treeTable.getTree();
    	TreeModel treeModel=tree.getModel();
    	
    	find=find.trim();
    	
    	TreeTableNode treeRoot = sn.getTtn();
    	if(treeRoot==null){
    		treeRoot=(TreeTableNode) treeModel.getRoot();
    	}
    	
    	SearchNode recBack = recSearch(find,0,treeRoot,treeModel);
    	if(recBack==null)
    	{
    		if(!prevNodes.isEmpty())
    		{
    			lastNode=null;
        		prevNodes.clear();
    			return search(find);
    		}
    		lastNode=null;
    		prevNodes.clear();
    		return false;
    	}
    	String[] nodeStrings=recBack.getPathStrings();

    	found=selectStrings(nodeStrings);

    	return found;
    }

	
	public void showSearchPanel(boolean show) {
		if (show) {
            if (searchPanel == null) {
                searchPanel = new SearchPanel(this, this,false,false);
                GridBagConstraints gbc = new GridBagConstraints();
                gbc.insets = new Insets(5, 5, 5, 5);
                gbc.fill = GridBagConstraints.HORIZONTAL;
                gbc.anchor = GridBagConstraints.CENTER;
                gbc.weightx = 0.10;
                gbc.weighty = 0.01;
                addCompItem(searchPanel, gbc, 0, 2, 2, 1);
                searchPanel.setFocus();
            }
        } else {
        	if(searchPanel!=null)
            {
        		getContentPane().remove(searchPanel);
            }
            searchPanel = null;
        }

        showFindPanelBox.setSelected(show);
        validate();
		
	}

	
	public void actionPerformed(ActionEvent evt) {
		 try {
	            Object EventSrc = evt.getSource();

	            if (EventSrc instanceof JMenuItem) {
	                String arg = evt.getActionCommand();
	                if (arg.equals("Show Find Panel")) {
	                    if (showFindPanelBox.isSelected()) {
	                        showSearchPanel(true);
	                    } else {
	                        showSearchPanel(false);
	                    }
	                } else {
	                    throw new ParaProfException("Menu system not implemented properly: " + arg);
	                }
	            }
	        } catch (Exception e) {
	            ParaProfUtils.handleException(e);
	        }
		
	}

	
	public void keyTyped(KeyEvent e) {
	}

	
	public void keyPressed(KeyEvent e) {
        if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_F) {
            showSearchPanel(true);
        }
    }

	
	public void keyReleased(KeyEvent e) {
	}

	
	String searchString="";
	
	public boolean setSearchString(String searchString) {
		if(!nexting){
			prevNodes.clear();
		}
		else{
			nexting=false;
		}
		this.searchString = searchString;

        if (searchString.length() == 0) { // reset
        	sn.setSearchRow(0);
        	sn.setTtn(null);
            repaint();
            return true;
        }

        if (!searchMatchCase) { // not matching case, so convert everything to uppercase for comparison
            searchString = searchString.toUpperCase();
        }

        boolean found = false;

//        if (searchUp) {
//        //TODO: Implement 'last' previous navigation here
//        } else 
        {

        			found=search(searchString);

        }

        if (!found) {
            repaint();
            return false;
        }

//        checkSearchStringVisibility();

        repaint();
        return true;
	}

	
	public void setSearchHighlight(boolean highlight) {
	
	}

	
	public void setSearchMatchCase(boolean matchCase) {
		searchMatchCase = matchCase;
        repaint();
		
	}

	
	public boolean searchNext() {
		nexting=true;
		if(lastNode!=null)
		{
			prevNodes.add(lastNode);
		}
		int sr = sn.getSearchRow();
		sn.setSearchRow(sr++);
        setSearchString(searchString);
        return false;
	}

	
	public boolean searchPrevious() {
		// TODO Implement previous navigation in search
		return false;
	}
}
